<?php
/**
 * Created by PhpStorm.
 * User: daniel
 * Date: 10.10.14
 * Time: 17:59
 */
namespace Cundd\PersistentObjectStore\Server\Handler;

use Cundd\PersistentObjectStore\Server\ValueObject\RequestInfo;

/**
 * Interface for classes that handle the actions from incoming requests
 *
 * @package Cundd\PersistentObjectStore\Server\Handler
 */
interface HandlerInterface {
	/**
	 * Invoked if no route is given (e.g. if the request path is empty)
	 *
	 * @param RequestInfo $requestInfo
	 * @return HandlerResultInterface
	 */
	public function noRoute(RequestInfo $requestInfo);

	/**
	 * Creates a new Data instance or Database with the given data for the given RequestInfo
	 *
	 * @param RequestInfo $requestInfo
	 * @param mixed $data
	 * @return HandlerResultInterface
	 */
	public function create(RequestInfo $requestInfo, $data);

	/**
	 * Read Data instances for the given RequestInfo
	 *
	 * @param RequestInfo $requestInfo
	 * @return HandlerResultInterface
	 */
	public function read(RequestInfo $requestInfo);

	/**
	 * Update a Data instance with the given data for the given RequestInfo
	 *
	 * @param RequestInfo $requestInfo
	 * @param mixed $data
	 * @return HandlerResultInterface
	 */
	public function update(RequestInfo $requestInfo, $data);

	/**
	 * Deletes a Data instance for the given RequestInfo
	 *
	 * @param RequestInfo $requestInfo
	 * @return HandlerResultInterface
	 */
	public function delete(RequestInfo $requestInfo);

	/**
	 * Action to display server statistics
	 *
	 * @param RequestInfo $requestInfo
	 * @return HandlerResultInterface
	 */
	public function getStatsAction(RequestInfo $requestInfo);

	/**
	 * Action to display all databases
	 *
	 * @param RequestInfo $requestInfo
	 * @return HandlerResultInterface
	 */
	public function getAllDbsAction(RequestInfo $requestInfo);
}